# Copyright 2005 by
#           Atte Timonen
#           Tarmo Toikkanen
#           Teemu Leinonen
#           Lily Diaz
#           Janne Pietarila
#
# Code snippets used from Fle3 (fle3.uiah.fi),
# credits go to the Fle3 Team.
#
# This file is part of the ImaNote product.
#
# ImaNote is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# ImaNote is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ImaNote; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Python
from PIL import Image, ImageDraw
from cStringIO import StringIO
import string
import os.path
import re
import time
# Zope
import DateTime
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, Persistent, package_home
from Acquisition import Implicit
import Acquisition
from AccessControl.Role import RoleManager
from OFS.Folder import Folder
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from CookieCrumbler import *
# Own
from common import *
from Users import Users
from Annotation import Annotation
from TiledImage import *
from AnnotationManager import AnnotationManager

def addFunction(self, my_id, title=None, REQUEST=None):
    """ """
    self._setObject(my_id, Made(my_id, title))

    if REQUEST:
      REQUEST.RESPONSE.redirect('/'.join((self.absolute_url(),my_id,
          'addImage.html')))
        #return self.manage_main(self, REQUEST)

addForm = PageTemplateFile('addTiledImage.html', globals())


zpt_files = ['index_html',
              'annotation_1.html',
              'annotation_2.html',
              'annotation_3.html',
              'annotation_edit.html',
              'annotation_edit_help.html',
              'annotations.xml',
              'addImage.html',
              'manageImages.html',
              'map.html',
              'search.html',
              'results.html',              
              'annotation_add.html',
              'login.html',
              'logout.html',
              'master.html',
              'addUser.html',
              'manageUsers.html',
              'getAnnotations.html',
              ]
ext_files = ['annotations.js',
             'wz_jsgraphics.js',
             'master.css',
             'utils.js',
             'annotations.css'
             ]

class Made(Folder, Implicit, Persistent, RoleManager, IDManager):
    """
    This is class for viewing large images (without annotations).
    """
    #FIXME add image management tab to ZMI
    addImage=PageTemplateFile('zpt/addImage.html', globals())
    manage_options=( Folder.manage_options + 
        ({'label' : 'Image Management', 'action' : 'addImage'},)
      )

    meta_type = 'ImaNote'
    security = ClassSecurityInfo()
    security.declareObjectPublic()

    def __init__(self, my_id, title):
        self.id = my_id
        self.title = title
        IDManager.__init__(self)

    security.declareProtected(PERM_MANAGE, 'addMap')
    def addMap(self, imageId, magnificationStep, imageFile, imageTitle='',\
        tileWidth=128, tileHeight=128, forceRgb=0):
      """Add a new map to this Made instance"""
      img=AnnotatedTiledImage(imageId, magnificationStep,\
          imageFile, imageTitle, tileWidth, tileHeight)
      self._setObject(imageId, img)
      #self.REQUEST.RESPONSE.redirect('/'.join((self.absolute_url(),imageId)))
      self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected(PERM_VIEW, 'getMap')
    def getMap(self, id):
      return self._getOb(id)

    security.declareProtected(PERM_VIEW, 'getMaps')
    def getMaps(self):
      """Return all maps"""
      return self.objectValues(AnnotatedTiledImage.meta_type)

    security.declareProtected(PERM_VIEW, 'chooseMap')
    def chooseMap(self, image):
      """Redirect to specified map"""
      self.REQUEST.RESPONSE.redirect(image)

    security.declarePrivate('manage_AfterAdd')
    def manage_afterAdd(self, item, container):
        """The rest of initialization code."""
        manage_addCC(self, 'cookiecrumbler')
        self.cookiecrumbler.auto_login_page='../login.html'
        self.cookiecrumbler.unauth_page='../login.html'
        self.cookiecrumbler.logout_page='../logout.html'
        self.manage_permission(PERM_EDIT, ('Authenticated',), 1)
        self.manage_permission(PERM_VIEW, ('Anonymous',), 1)
        self.manage_permission(PERM_MANAGE, ('Manager',), 1)
        self.manage_addFolder('annotations', 'Image annotations')
        self._setObject('users', Users('users', 'Users / '+self.title))
        self._setObject('img',Folder())

        # Read Zpt files from the file system and add the to ZODB.
        self.reload_zpt()
        self.reload_images()

    security.declareProtected(PERM_MANAGE, 'reload_zpt')
    def reload_zpt(self):
        """Reload zpt files from the file system."""
        directory = os.path.join(package_home(globals()), 'zpt')
        parent = self.aq_inner.aq_parent
        for name in zpt_files:
            f = open(os.path.join(directory, name))
            content = f.read()
            f.close()
            if reallyHasattr(self, name):
              self._delObject(name)
            self._setObject(name,ZopePageTemplate(name,content))
        for name in ext_files:
            f = open(os.path.join(directory, name))
            content = f.read()
            f.close()
            if reallyHasattr(self, name):
              self._delObject(name)
            self.manage_addFile(name, content)
        get_transaction().commit()
#FIXME Setting permissions works only when reload_zpt is called from manage_afterAdd
        #getattr(self, 'annotation_add.html').manage_permission('View', ('Authenticated',), 0)
        #getattr(self, 'annotation_1.html').manage_permission('View', ('Authenticated',), 0)
        #getattr(self, 'addImage.html').manage_permission('View', ('Authenticated',), 0)
        return 'Zpt files reloaded.'
      
    def reload_images(self):
        """Reload images from the file system."""
        directory = os.path.join(package_home(globals()), 'www')
        parent = self.aq_inner.aq_parent
        for filename in os.listdir(directory):
          n = filename.index('.')
          if n==0: continue
          data = open(os.path.join(directory, filename))
          content=data.read()
          data.close()
          new_id = filename[:n]
          if new_id in self.img.objectIds():
            self.img.manage_delObjects(new_id)
          self.img.manage_addImage(new_id,content,'')
          
        return 'Images reloaded.'

    def getUser(self):
      """Return name of current user"""
      try:
        return str(self.REQUEST.AUTHENTICATED_USER)
      except AttributeError:
        return 'Anonymous User'


    def isUserAuthenticated(self):
      """Return whether current user is authenticated"""
      try:
        return str(self.REQUEST.AUTHENTICATED_USER)!='Anonymous User'
      except AttributeError:
        return False


    security.declareProtected(PERM_EDIT, 'requireEditPermission')
    def requireEditPermission(self):
      """Require user to log in when this method is called"""
      return 1

    security.declareProtected(PERM_MANAGE, 'manage_images_handler')
    def manage_images_handler(self, save=''):
      """Form handler for image management"""
      maps=self.getMaps()
      annosShown={}
      annosEditable={}
      for i in maps:
        if hasattr(self.REQUEST, '%sshow_annotations' % (i.getId())):
          tmp=eval('self.REQUEST.%sshow_annotations' % (i.getId()))
          annosShown[i.getId()]=tmp.replace(',','').split(' ')
          tmp=eval('self.REQUEST.%sedit_annotations' % (i.getId()))
          annosEditable[i.getId()]=tmp.replace(',','').split(' ')
    
      #Handle annotation managers
      annoManagers=self.annotations.objectValues()
      for i in annoManagers:
        if hasattr(self.REQUEST, i.getId()+'deleteManager') and\
            getattr(self.REQUEST, i.getId()+'deleteManager')=='on':
              self.annotations._delObject(i.getId())
              maps=self.getMaps()
              for j in maps:
                if i.getId() in annosShown[j.getId()] and \
                    (not hasattr(self.REQUEST, 'newAnnoSet') 
                    or self.REQUEST.newAnnoSet!=i.getId()):
                  annosShown[j.getId()].remove(i.getId())
                if i.getId() in annosEditable[j.getId()] and \
                    (not hasattr(self.REQUEST, 'newAnnoSet') 
                    or self.REQUEST.newAnnoSet!=i.getId()):
                  annosEditable[j.getId()].remove(i.getId())

      if hasattr(self.REQUEST, 'newAnnoSet') and self.REQUEST.newAnnoSet!='':
        #Add new annotation set
        self.annotations._setObject(self.REQUEST.newAnnoSet,\
            AnnotationManager(self.REQUEST.newAnnoSet))

      #Handle images
      for i in maps:
        if hasattr(self.REQUEST, i.getId()+'delete') and\
            getattr(self.REQUEST, i.getId()+'delete')=='on':
          self._delObject(i.getId())
          continue
        #title
        if hasattr(self.REQUEST, i.getId()+'title'):
          i.title=eval('self.REQUEST.%stitle' % i.getId())

        #colors
        if hasattr(self.REQUEST, i.getId()+'color_1'):
          for j in ['color_1', 'color_2']:
            color=eval('self.REQUEST.%s%s' % (i.getId(), j))
            try:
              if len(color)<6:
                raise ValueError, ''
              int('0x'+color,16)
              exec('i.%s=self.REQUEST.%s%s' % (j, i.getId(), j))
            except ValueError:
              raise ValueError, 'Invalid annotation color - Color has to be 6-digit hex number'
            
        #Annotation minimum size
        try:
          if hasattr(self.REQUEST, i.getId()+'anno_min_size'):
            minsize=eval('int(self.REQUEST.%sanno_min_size)' %(i.getId()))
            if minsize<=0:
              raise ValueError
            i.anno_min_size=minsize
        except ValueError:
          raise ValueError, 'Invalid annotation minimum size - Minimum size has to be positive integer'

        #shown and editable annotations
        for j in annosShown[i.getId()]:
          if j not in self.annotations.objectIds() and j!='':
            raise ValueError, 'The annotation set %s does not exist' % j
        i.show_annotations=tuple(annosShown[i.getId()])
        for j in annosEditable[i.getId()]:
          if j not in self.annotations.objectIds() and j!='':
            raise ValueError, 'The annotation set %s does not exist' % j
        i.edit_annotations=tuple(annosEditable[i.getId()])

        #editing
        if hasattr(self.REQUEST, i.getId()+'edit_disabled'):
          i.edit_disabled=int(eval('self.REQUEST.%sedit_disabled' % (i.getId())))


      self.REQUEST.RESPONSE.redirect('manageImages.html')

    security.declareProtected(PERM_VIEW, 'getTupleAsTokens')
    def getTupleAsTokens(self, tup):
      """Convert a tuple into a token string"""
      return str(tup).replace('(', '').replace(')', '').replace("'","").replace(',','')
    
          
        
InitializeClass(Made)
